package jdk.internal.org.objectweb.asm.tree;

import java.util.Iterator;
import jdk.internal.org.objectweb.asm.MethodVisitor;
import jdk.internal.org.objectweb.asm.FieldVisitor;
import jdk.internal.org.objectweb.asm.TypePath;
import jdk.internal.org.objectweb.asm.AnnotationVisitor;
import java.util.Arrays;
import java.util.ArrayList;
import jdk.internal.org.objectweb.asm.Attribute;
import java.util.List;
import jdk.internal.org.objectweb.asm.ClassVisitor;

public class ClassNode extends ClassVisitor
{
    public int version;
    public int access;
    public String name;
    public String signature;
    public String superName;
    public List<String> interfaces;
    public String sourceFile;
    public String sourceDebug;
    public String outerClass;
    public String outerMethod;
    public String outerMethodDesc;
    public List<AnnotationNode> visibleAnnotations;
    public List<AnnotationNode> invisibleAnnotations;
    public List<TypeAnnotationNode> visibleTypeAnnotations;
    public List<TypeAnnotationNode> invisibleTypeAnnotations;
    public List<Attribute> attrs;
    public List<InnerClassNode> innerClasses;
    public List<FieldNode> fields;
    public List<MethodNode> methods;
    
    public ClassNode() {
        this(327680);
        if (this.getClass() != ClassNode.class) {
            throw new IllegalStateException();
        }
        return;
    }
    
    public ClassNode(int api) {
        super(api);
        this.interfaces = new ArrayList();
        this.innerClasses = new ArrayList();
        this.fields = new ArrayList();
        this.methods = new ArrayList();
    }
    
    @Override
    public void visit(int version, int access, String name, String signature, String superName, String[] a) {
        this.version = version;
        this.access = access;
        this.name = name;
        this.signature = signature;
        this.superName = superName;
        if (a != null) {
            this.interfaces.addAll(Arrays.asList(a));
        }
    }
    
    @Override
    public void visitSource(String sourceFile, String sourceDebug) {
        this.sourceFile = sourceFile;
        this.sourceDebug = sourceDebug;
    }
    
    @Override
    public void visitOuterClass(String outerClass, String outerMethod, String outerMethodDesc) {
        this.outerClass = outerClass;
        this.outerMethod = outerMethod;
        this.outerMethodDesc = outerMethodDesc;
    }
    
    @Override
    public AnnotationVisitor visitAnnotation(String s, boolean b) {
        AnnotationNode annotationNode = new AnnotationNode(s);
        if (b) {
            if (this.visibleAnnotations == null) {
                this.visibleAnnotations = new ArrayList(1);
            }
            this.visibleAnnotations.add(annotationNode);
            return annotationNode;
        }
        else {
            if (this.invisibleAnnotations == null) {
                this.invisibleAnnotations = new ArrayList(1);
            }
            this.invisibleAnnotations.add(annotationNode);
            return annotationNode;
        }
    }
    
    @Override
    public AnnotationVisitor visitTypeAnnotation(int n, TypePath typePath, String s, boolean b) {
        TypeAnnotationNode typeAnnotationNode = new TypeAnnotationNode(n, typePath, s);
        if (b) {
            if (this.visibleTypeAnnotations == null) {
                this.visibleTypeAnnotations = new ArrayList(1);
            }
            this.visibleTypeAnnotations.add(typeAnnotationNode);
            return typeAnnotationNode;
        }
        else {
            if (this.invisibleTypeAnnotations == null) {
                this.invisibleTypeAnnotations = new ArrayList(1);
            }
            this.invisibleTypeAnnotations.add(typeAnnotationNode);
            return typeAnnotationNode;
        }
    }
    
    @Override
    public void visitAttribute(Attribute attribute) {
        if (this.attrs == null) {
            this.attrs = new ArrayList(1);
        }
        this.attrs.add(attribute);
    }
    
    @Override
    public void visitInnerClass(String s, String s2, String s3, int n) {
        this.innerClasses.add(new InnerClassNode(s, s2, s3, n));
    }
    
    @Override
    public FieldVisitor visitField(int n, String s, String s2, String s3, Object o) {
        FieldNode fieldNode = new FieldNode(n, s, s2, s3, o);
        this.fields.add(fieldNode);
        return fieldNode;
    }
    
    @Override
    public MethodVisitor visitMethod(int n, String s, String s2, String s3, String[] array) {
        MethodNode methodNode = new MethodNode(n, s, s2, s3, array);
        this.methods.add(methodNode);
        return methodNode;
    }
    
    @Override
    public void visitEnd() {
    }
    
    public void check(int n) {
        if (n == 262144) {
            if (this.visibleTypeAnnotations != null && this.visibleTypeAnnotations.size() > 0) {
                throw new RuntimeException();
            }
            if (this.invisibleTypeAnnotations != null && this.invisibleTypeAnnotations.size() > 0) {
                throw new RuntimeException();
            }
            Iterator<FieldNode> iterator = this.fields.iterator();
            while (iterator.hasNext()) {
                ((FieldNode)iterator.next()).check(n);
            }
            Iterator<MethodNode> iterator2 = this.methods.iterator();
            while (iterator2.hasNext()) {
                ((MethodNode)iterator2.next()).check(n);
            }
        }
    }
    
    public void accept(ClassVisitor classVisitor) {
        String[] interfaces = new String[this.interfaces.size()];
        this.interfaces.toArray(interfaces);
        classVisitor.visit(this.version, this.access, this.name, this.signature, this.superName, interfaces);
        if (this.sourceFile != null || this.sourceDebug != null) {
            classVisitor.visitSource(this.sourceFile, this.sourceDebug);
        }
        if (this.outerClass != null) {
            classVisitor.visitOuterClass(this.outerClass, this.outerMethod, this.outerMethodDesc);
        }
        for (int n = (this.visibleAnnotations == null) ? 0 : this.visibleAnnotations.size(), i = 0; i < n; ++i) {
            AnnotationNode annotationNode = (AnnotationNode)this.visibleAnnotations.get(i);
            annotationNode.accept(classVisitor.visitAnnotation(annotationNode.desc, true));
        }
        for (int n2 = (this.invisibleAnnotations == null) ? 0 : this.invisibleAnnotations.size(), j = 0; j < n2; ++j) {
            AnnotationNode annotationNode2 = (AnnotationNode)this.invisibleAnnotations.get(j);
            annotationNode2.accept(classVisitor.visitAnnotation(annotationNode2.desc, false));
        }
        for (int n3 = (this.visibleTypeAnnotations == null) ? 0 : this.visibleTypeAnnotations.size(), k = 0; k < n3; ++k) {
            TypeAnnotationNode typeAnnotationNode = (TypeAnnotationNode)this.visibleTypeAnnotations.get(k);
            typeAnnotationNode.accept(classVisitor.visitTypeAnnotation(typeAnnotationNode.typeRef, typeAnnotationNode.typePath, typeAnnotationNode.desc, true));
        }
        for (int n4 = (this.invisibleTypeAnnotations == null) ? 0 : this.invisibleTypeAnnotations.size(), l = 0; l < n4; ++l) {
            TypeAnnotationNode typeAnnotationNode2 = (TypeAnnotationNode)this.invisibleTypeAnnotations.get(l);
            typeAnnotationNode2.accept(classVisitor.visitTypeAnnotation(typeAnnotationNode2.typeRef, typeAnnotationNode2.typePath, typeAnnotationNode2.desc, false));
        }
        for (int n5 = (this.attrs == null) ? 0 : this.attrs.size(), n6 = 0; n6 < n5; ++n6) {
            classVisitor.visitAttribute((Attribute)this.attrs.get(n6));
        }
        for (int n7 = 0; n7 < this.innerClasses.size(); ++n7) {
            ((InnerClassNode)this.innerClasses.get(n7)).accept(classVisitor);
        }
        for (int n8 = 0; n8 < this.fields.size(); ++n8) {
            ((FieldNode)this.fields.get(n8)).accept(classVisitor);
        }
        for (int n9 = 0; n9 < this.methods.size(); ++n9) {
            ((MethodNode)this.methods.get(n9)).accept(classVisitor);
        }
        classVisitor.visitEnd();
    }
}
